package me.msoucy.ptures.model

class Engine(vararg val creatures : Pair<Creature, Int>) {
    init {
        assert(creatures.isNotEmpty())
    }

    var currentCreature = 0

    val attacker : Creature get() {
        return creatures[currentCreature].first
    }

    fun resolveTurn() {
        val activeCreatures = creatures.withIndex()
                .filter { (_, c) -> !c.first.hasStatus<KnockedOut>() }
                .sortedBy { (_, c) -> c.first.spd }
        // Get the moves each creature will use this turn
        val moves = activeCreatures.map { (_, c) -> Pair(c.first.skills[0], 0) }
        val wasHit = mutableListOf<Creature>()
        for ((i, c) in activeCreatures) {
            // Resolve move
            val (skill, target) = moves[i]
            currentCreature = i
            for (step in skill.damageSteps) {
                val targets = getTargetList(step.target, currentCreature, target)
                for (t in targets) {
                    val (targetCreature, _) = creatures[t]
                    if (attacker.hits(targetCreature, step)) {
                        targetCreature.apply(step, c.first)
                        wasHit.add(targetCreature)
                    }
                }
            }
            for(creature in wasHit) {
                for(step in skill.statusSteps) {
                    step(creature)
                }
            }
        }
    }

    private fun getTargetList(target : Target, active : Int, selected : Int) : List<Int> {
        return when(target) {
            Target.Self -> listOf(active)
            Target.Selected -> listOf(selected)
            Target.Others -> creatures.indices.filter { it != active }
            Target.Opponents -> creatures.indices.filter { creatures[it].second != creatures[active].second }
            Target.All -> creatures.indices.toList()
        }
    }
}